import bcrypt
from apps.auth.errors import NotFoundAuthError, raiseCredentialsAuthError
from apps.auth.service import AuthService
from superdesk import get_resource_service


class DbAuthService(AuthService):

    def authenticate(self, credentials):
        user = get_resource_service('auth_users').find_one(req=None, username=credentials.get('username'))
        if not user:
            raise NotFoundAuthError()

        password = credentials.get('password').encode('UTF-8')
        hashed = user.get('password').encode('UTF-8')

        if not (password and hashed):
            raiseCredentialsAuthError(credentials)

        try:
            rehashed = bcrypt.hashpw(password, hashed)
            if hashed != rehashed:
                raiseCredentialsAuthError(credentials)
        except ValueError:
            raiseCredentialsAuthError(credentials)

        return user



from hachoir.stream import InputIOStream
from hachoir.parser import guessParser
from hachoir.metadata import extractMetadata
from flask import json
import logging


logger = logging.getLogger(__name__)


def get_meta(filestream):
    metadata = {}

    try:
        filestream.seek(0)
        stream = InputIOStream(filestream, None, tags=[])
        parser = guessParser(stream)
        if not parser:
            return metadata

        tags = extractMetadata(parser).exportPlaintext(human=False, line_prefix='')
        for text in tags:
            try:
                json.dumps(text)
                key, value = text.split(':', maxsplit=1)
                key, value = key.strip(), value.strip()
                if key and value:
                    metadata.update({key: value})
            except Exception as ex:
                logger.exception(ex)
    except Exception as ex:
        logger.exception(ex)
        return metadata
    return metadata



class BaseComponent():
    '''
    This is a basic interface for defining components. The only requirement is
    to implement the name method that uniquely identifies a component. It
    should also define other methods that implement the component functionality.
    '''

    @classmethod
    def name(cls):
        raise NotImplementedError()



import logging
from eve.auth import TokenAuth
from flask import current_app as app, request
import flask
from apps.auth.errors import AuthRequiredError, ForbiddenError
from superdesk.resource import Resource
from superdesk import get_resource_service


logger = logging.getLogger(__name__)


class AuthUsersResource(Resource):
    """ This resource is for authentication only.

    On users `find_one` never returns a password due to the projection.
    """
    datasource = {'source': 'users'}
    schema = {
        'username': {
            'type': 'string',
        },
        'password': {
            'type': 'string',
        },
        'status': {
            'type': 'string'
        }
    }
    item_methods = []
    resource_methods = []
    internal_resource = True


class AuthResource(Resource):
    schema = {
        'username': {
            'type': 'string',
            'required': True
        },
        'password': {
            'type': 'string',
            'required': True
        },
        'token': {
            'type': 'string'
        },
        'user': Resource.rel('users', True)
    }
    resource_methods = ['POST']
    item_methods = ['GET']
    public_methods = ['POST']
    extra_response_fields = ['user', 'token', 'username']


class SuperdeskTokenAuth(TokenAuth):
    """Superdesk Token Auth"""

    method_map = {
        'get': 'read',
        'put': 'write',
        'patch': 'write',
        'post': 'write',
        'delete': 'write',
    }

    def check_permissions(self, resource, method, user):
        if not user:
            return True

        # is the operation against the user record of the current user
        if request.view_args.get('_id') == str(user['_id']):
            # no user is allowed to delete their own user
            if method.lower() == 'delete':
                raise ForbiddenError
            else:
                return True

        # We allow all reads
        perm_method = self.method_map[method.lower()]
        if perm_method is 'read':
            return True

        # Read the user type
        user_type = user.get('user_type')

        # We allow all writes to administrators
        if user_type == 'administrator':
            return True

        # Only administrators can write to those resources in this list
        if resource in {'users', 'roles'}:
            raise ForbiddenError()

        # Get the list of roles belonging to this user
        roles = user.get('roles')
        if roles is not None:
            for role_id in roles:
                # check the permissions for each role
                while role_id:
                    role = get_resource_service('roles').find_one(_id=role_id, req=None) or {}
                    perm = role.get('permissions', {})
                    if perm.get(resource, {}).get(perm_method, False):
                        return True
                    role_id = role.get('extends')

        # If we didn't return True so far then user is not authorized
        raise ForbiddenError()

    def check_auth(self, token, allowed_roles, resource, method):
        """Check if given token is valid"""
        auth_token = get_resource_service('auth').find_one(token=token, req=None)
        if auth_token:
            user_id = str(auth_token['user'])
            flask.g.user = get_resource_service('users').find_one(req=None, _id=user_id)
            return self.check_permissions(resource, method, flask.g.user)

    def authorized(self, allowed_roles, resource, method):
        """Ignores auth on home endpoint."""
        if not resource:
            return True
        if app.debug and request.args.get('skip_auth'):
            return True
        return super(SuperdeskTokenAuth, self).authorized(allowed_roles, resource, method)

    def authenticate(self):
        """ Returns 401 response with CORS headers."""
        raise AuthRequiredError()




import blinker


def connect(signal, subscriber):
    """Connect to signal"""
    blinker.signal(signal).connect(subscriber)


def send(signal, sender, **kwargs):
    """Send signal"""
    return blinker.signal(signal).send(sender, **kwargs)


def proxy_resource_signal(action, app):
    def handle(resource, documents):
        docs = documents
        if '_items' in documents:
            docs = documents['_items']
        send(action, app.data, docs=docs)
        send('%s:%s' % (action, resource), app.data, docs=docs)
    return handle


def proxy_item_signal(action, app):
    def handle(resource, document):
        send(action, app.data, resource=resource, docs=[document])
        send('%s:%s' % (action, resource), app.data, docs=[document])
    return handle



#!/usr/bin/env python

from setuptools import setup, find_packages

with open('README.md') as f:
    LONG_DESCRIPTION = f.read()

setup(
    name='Superdesk-Server',
    version='0.1.5-beta',
    description='Superdesk REST API server',
    long_description=LONG_DESCRIPTION,
    author='petr jasek',
    author_email='petr.jasek@sourcefabric.org',
    url='https://github.com/superdesk/superdesk-server',
    license='GPLv3',
    platforms=['any'],
    packages=find_packages(),
    install_requires=[
        'Eve>=0.4',
        'Eve-Elastic>=0.1.13',
        'Flask>=0.10,<0.11',
        'Flask-Mail>=0.9.0,<0.10',
        'Flask-Script==2.0.5,<2.1',
        'Flask-PyMongo>=0.3.0',
        'autobahn[asyncio]>=0.8.15',
        'celery[redis]>=3.1.13',
        'bcrypt>=1.0.2',
        'blinker>=1.3',
    ],
    scripts=['settings.py', 'app.py', 'wsgi.py', 'ws.py', 'manage.py'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ]
)



''' Amazon media storage module'''
from eve.io.media import MediaStorage
import tinys3
from superdesk import SuperdeskError
import logging
import json
from io import BytesIO

logger = logging.getLogger(__name__)


class AmazonObjectWrapper(BytesIO):

    def __init__(self, stream_generator, content_type, length, name, metadata):
        super().__init__()
        for bytes in stream_generator:
            self.write(bytes)
        self.seek(0)
        self.content_type = content_type
        self.length = int(length)
        self.name = name
        self.metadata = metadata


class AmazonMediaStorage(MediaStorage):

    def __init__(self, app=None):
        super().__init__(app)
        username, api_key, endpoint = self.read_from_config()
        self.conn = tinys3.Connection(username, api_key, tls=True, endpoint=endpoint)
        self.user_metadata_header = 'x-amz-meta-'

    def read_from_config(self):
        if 'AMAZON_REGION' in self.app.config:
            region = self.app.config['AMAZON_REGION']
        else:
            region = 's3'
        username = self.app.config['AMAZON_ACCESS_KEY_ID']
        api_key = self.app.config['AMAZON_SECRET_ACCESS_KEY']
        self.container_name = self.app.config['AMAZON_CONTAINER_NAME']
        endpoint = '%s.amazonaws.com' % region
        return username, api_key, endpoint

    def get_bucket_objects(self, marker=None, bucket=None):
        '''
        Fetch the objects available in the specified bucket.
        '''
        bucket = bucket or self.container_name
        params = {'marker': marker}
        rv = self.conn.get_bucket_objects(bucket=bucket, extra_params=params)
        if rv.status_code not in (200, 201):
            message = 'Retrieving the list of files from bucket %s failed' % bucket
            raise SuperdeskError(payload=message)
        content = rv.content.decode('UTF-8')
        return content

    def get(self, id_or_filename):
        """ Opens the file given by name or unique id. Note that although the
        returned file is guaranteed to be a File object, it might actually be
        some subclass. Returns None if no file was found.
        """
        id_or_filename = str(id_or_filename)
        found, obj = self._check_exists(id_or_filename)
        if found:
            metadata = self.extract_metadata_from_headers(obj.headers)
            return AmazonObjectWrapper(obj, obj.headers['content-type'], obj.headers['content-length'], id_or_filename,
                                       metadata)
        return None

    def extract_metadata_from_headers(self, request_headers):
        headers = {}
        for key, value in request_headers.items():
            if self.user_metadata_header in key:
                new_key = key.split(self.user_metadata_header)[1]
                if(value):
                    try:
                        headers[new_key] = json.loads(value)
                    except Exception as ex:
                        logger.exception(ex)
        return headers

    def update_metadata(self, key, metadata):
        if not metadata:
            return
        metadata = self.transform_metadata_to_amazon_format(metadata)
        res = self.conn.update_metadata(key, metadata, bucket=self.container_name)
        if res.status_code not in (200, 201):
            payload = 'Updating metadata for file %s failed' % key
            raise SuperdeskError(payload=payload)

    def transform_metadata_to_amazon_format(self, metadata):
        if not metadata:
            return {}
        file_metadata = {}
        for key, value in metadata.items():
            new_key = self.user_metadata_header + key
            file_metadata[new_key] = value
        return file_metadata

    def put(self, content, filename=None, content_type=None, metadata=None):
        """ Saves a new file using the storage system, preferably with the name
        specified. If there already exists a file with this name name, the
        storage system may modify the filename as necessary to get a unique
        name. Depending on the storage system, a unique id or the actual name
        of the stored file will be returned. The content type argument is used
        to appropriately identify the file when it is retrieved.
        """
        logger.debug('Going to save media file with %s ' % filename)
        found, existing_file = self._check_exists(filename, raise_error=False)
        if found:
            return filename

        try:
            file_metadata = self.transform_metadata_to_amazon_format(metadata)
            res = self.conn.upload(filename, content, self.container_name, content_type=content_type,
                                   headers=file_metadata)
            if res.status_code not in (200, 201):
                raise SuperdeskError(payload='Uploading file to amazon S3 failed')
            return filename
        except Exception as ex:
            logger.exception(ex)
            raise

    def delete(self, id_or_filename):
        id_or_filename = str(id_or_filename)
        del_res = self.conn.delete(id_or_filename, self.container_name)
        logger.debug('Amazon S3 file deleted %s with status' % id_or_filename, del_res.status_code)

    def exists(self, id_or_filename):
        """ Returns True if a file referenced by the given name or unique id
        already exists in the storage system, or False if the name is available
        for a new file.
        """
        id_or_filename = str(id_or_filename)
        found, _ = self._check_exists(id_or_filename)
        return found

    def _check_exists(self, id_or_filename, raise_error=True):
        try:
            obj = self.conn.get(id_or_filename, self.container_name)
            if obj.status_code not in (200, 201) and raise_error:
                message = 'Retrieving file %s from amazon failed' % id_or_filename
                raise SuperdeskError(payload=message)
            return (True, obj)
        except Exception as ex:
            if raise_error:
                logger.exception(ex)
            # File not found
            return (False, None)



from eve.io.mongo.media import GridFSMediaStorage
import logging
import json
from bson import ObjectId
from gridfs import GridFS


logger = logging.getLogger(__name__)


class SuperdeskGridFSMediaStorage(GridFSMediaStorage):

    def get(self, _id):
        logger.debug('Getting media file with id= %s' % _id)
        if isinstance(_id, str):
            _id = ObjectId(_id)
        media_file = super().get(_id)
        if media_file and media_file.metadata:
            for k, v in media_file.metadata.items():
                media_file.metadata[k] = json.loads(v)
        return media_file

    def put(self, content, filename=None, content_type=None, metadata=None):
        _id = self.fs().put(content, content_type=content_type, filename=filename, metadata=metadata)
        logger.debug('Saved  media file with id= %s' % _id)
        return _id

    def fs(self):
        if self._fs is None:
            self._fs = GridFS(self.app.data.mongo.driver.db)
        return self._fs



'''
Created on May 29, 2014

@author: ioan
'''


import redis
import arrow
import superdesk
from bson import ObjectId
from celery import Celery
from kombu.serialization import register
from eve.io.mongo import MongoJSONEncoder
from eve.utils import str_to_date
from flask import json


celery = Celery(__name__)
TaskBase = celery.Task


def try_cast(v):
    try:
        str_to_date(v)  # try if it matches format
        return arrow.get(v).datetime  # return timezone aware time
    except:
        try:
            return ObjectId(v)
        except:
            return v


def cast_item(o):
    with superdesk.app.app_context():
        for k, v in o.items():
            if isinstance(v, dict):
                cast_item(v)
            else:
                o[k] = try_cast(v)


def loads(s):
    o = json.loads(s)

    if not o.get('args', None):
        o['args'] = []

    if not o.get('kwargs', None):
        o['kwargs'] = {}

    for v in o['args']:
        if isinstance(v, dict):
            cast_item(v)

    kwargs = o['kwargs']
    for k, v in kwargs.items():
        if isinstance(v, str):
            kwargs[k] = try_cast(v)

        if isinstance(v, list):
            kwargs[k] = [try_cast(val) for val in v]

        if isinstance(v, dict):
            cast_item(v)

    return o


def dumps(o):
    with superdesk.app.app_context():
        return MongoJSONEncoder().encode(o)


register('eve/json', dumps, loads, content_type='application/json')


class AppContextTask(TaskBase):
    abstract = True
    serializer = 'eve/json'

    def __call__(self, *args, **kwargs):
        with superdesk.app.app_context():
            return super().__call__(*args, **kwargs)

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        try:
            superdesk.app.sentry.captureException()
        except:
            pass


celery.Task = AppContextTask


def init_celery(app):
    celery.conf.update(app.config)
    app.celery = celery


def add_subtask_to_progress(task_id):
    return _update_subtask_progress(task_id, total=True)


def finish_subtask_from_progress(task_id):
    return _update_subtask_progress(task_id, current=True)


def finish_task_for_progress(task_id):
    return _update_subtask_progress(task_id, done=True)


def update_key(redis_db, key, flag):
    if flag:
        crt_value = redis_db.incr(key)
    else:
        crt_value = redis_db.get(key)

    if crt_value:
        crt_value = int(crt_value)
    else:
        crt_value = 0

    return crt_value


def _update_subtask_progress(task_id, current=None, total=None, done=None):
    redis_db = redis.from_url(celery.conf['CELERY_RESULT_BACKEND'])

    current_key = 'current_%s' % task_id
    total_key = 'total_%s' % task_id
    done_key = 'done_%s' % task_id

    crt_current = update_key(redis_db, current_key, current)
    crt_total = update_key(redis_db, total_key, total)
    crt_done = update_key(redis_db, done_key, done)

    if crt_done and crt_current == crt_total:
        redis_db.delete(current_key)
        redis_db.delete(crt_total)
        redis_db.delete(done_key)

    return task_id, crt_current, crt_total



"""IPTC module"""

import os
from superdesk import json


def load_codes(filename):
    with open(filename, 'r') as f:
        codes = json.load(f)
        return codes


dirname = os.path.dirname(os.path.realpath(__file__))
data_subject_codes = os.path.join(dirname, 'data', 'subject_codes.json')
subject_codes = load_codes(data_subject_codes)



'''
Created on May 23, 2014

@author: Ioan v. Pocol
'''

from celery.canvas import chord
import flask

import superdesk
from superdesk.utc import utc, utcnow
from superdesk.celery_app import celery, finish_task_for_progress,\
    finish_subtask_from_progress, add_subtask_to_progress
from celery.result import AsyncResult
from flask.globals import current_app as app
from superdesk.upload import url_for_media
from superdesk.media.media_operations import download_file_from_url,\
    process_file
from superdesk.resource import Resource
from celery.exceptions import Ignore
from celery import states
from .common import facets
import traceback
from celery.utils.log import get_task_logger
from superdesk.services import BaseService


logger = get_task_logger(__name__)


def update_status(task_id, current, total):
    if current is None:
        current = 0
    archive_item.update_state(task_id, state='PROGRESS', meta={'current': int(current), 'total': int(total)})


def raise_fail(task_id, message):
    archive_item.update_state(task_id, state=states.FAILURE, meta={'error': message})
    # by raising Ignore the processing will stop(no retries) and the result is not updated anymore
    raise Ignore()


def import_rendition(guid, rendition_name, href, extract_metadata):
    archive = superdesk.get_resource_service('archive').find_one(req=None, guid=guid)
    if not archive:
        msg = 'No document found in the media archive with this ID: %s' % guid
        raise superdesk.SuperdeskError(payload=msg)

    if rendition_name not in archive['renditions']:
        payload = 'Invalid rendition name %s' % rendition_name
        raise superdesk.SuperdeskError(payload=payload)

    updates = {}
    metadata = None

    content, filename, content_type = download_file_from_url(href)
    if extract_metadata:
        file_type, ext = content_type.split('/')
        metadata = process_file(content, file_type)

    file_guid = app.media.put(content, filename, content_type, metadata)

    # perform partial update
    updates['renditions.' + rendition_name + '.href'] = url_for_media(file_guid)
    updates['renditions.' + rendition_name + '.media'] = file_guid
    result = superdesk.get_resource_service('archive').patch(guid, updates=updates)

    return result


@celery.task(bind=True, max_retries=3)
def archive_media(self, task_id, guid, href):
    try:
        if not self.request.retries:
            update_status(*add_subtask_to_progress(task_id))
        import_rendition(guid, 'baseImage', href, True)
        update_status(*finish_subtask_from_progress(task_id))
    except Exception:
        raise self.retry(countdown=2)


@celery.task(bind=True, max_retries=3)
def archive_rendition(self, task_id, guid, name, href):
    try:
        if not self.request.retries:
            update_status(*add_subtask_to_progress(task_id))
        import_rendition(guid, name, href, False)
        update_status(*finish_subtask_from_progress(task_id))
    except Exception:
        raise self.retry(countdown=2)


@celery.task()
def update_item(result, is_main_task, task_id, guid):
    if is_main_task:
        update_status(*finish_task_for_progress(task_id))


@celery.task(bind=True, max_retries=3)
def archive_item(self, guid, provider_id, user, task_id=None):
    try:
        # For CELERY_ALWAYS_EAGER=True the current request context is
        # empty but already initialized one is on request_stack
        if app.config['CELERY_ALWAYS_EAGER']:
            self.request_stack.pop()

        crt_task_id = self.request.id
        if not task_id:
            task_id = crt_task_id

        if not self.request.retries:
            update_status(*add_subtask_to_progress(task_id))

        provider = superdesk.get_resource_service('ingest_providers').find_one(req=None, _id=provider_id)
        if provider is None:
            message = 'For ingest with guid= %s, failed to retrieve provider with _id=%s' % (guid, provider_id)
            raise_fail(task_id, message)
        service_provider = superdesk.io.providers[provider.get('type')]
        service_provider.provider = provider

        item = None
        old_item = False
        try:
            items = service_provider.get_items(guid)
        except LookupError:
            ingest_doc = superdesk.get_resource_service('ingest').find_one(req=None, _id=guid)
            if not ingest_doc:
                message = 'Not found the ingest with guid: %s for provider %s' % (guid, provider.get('type'))
                raise_fail(task_id, message)
            else:
                old_item = True
                ingest_doc.pop('_id')
                items = [ingest_doc]
        except Exception:
            raise self.retry(countdown=2)

        for item_it in items:
            if 'guid' in item_it and item_it['guid'] == guid:
                item = item_it
                break

        if item is None:
            message = 'Returned ingest but not found the ingest with guid: %s for provider %s' \
                      % (guid, provider.get('type'))
            raise_fail(task_id, message)

        if not old_item:
            item['created'] = item['firstcreated'] = utc.localize(item['firstcreated'])
            item['updated'] = item['versioncreated'] = utc.localize(item['versioncreated'])
        superdesk.get_resource_service('archive').patch(guid, item)

        tasks = []
        for group in item.get('groups', []):
            for ref in group.get('refs', []):
                if 'residRef' in ref:
                    doc = {'guid': ref.get('residRef'), 'ingest_provider': provider_id,
                           'user': user, 'task_id': crt_task_id}

                    archived_doc = superdesk.get_resource_service('archive').find_one(req=None, guid=doc.get('guid'))
                    # check if task already started
                    if not archived_doc:
                        doc.setdefault('_id', doc.get('guid'))
                        superdesk.get_resource_service('archive').post([doc])
                    elif archived_doc.get('task_id') == crt_task_id:
                        # it is a retry so continue
                        archived_doc.update(doc)
                        superdesk.get_resource_service('archive').patch(archived_doc.get('_id'), archived_doc)
                    else:
                        # there is a cyclic dependency, skip it
                        continue

                    ingest_set_archived(doc.get('guid'))
                    tasks.append(archive_item.s(ref['residRef'], provider.get('_id'), user, task_id))

        for rendition in item.get('renditions', {}).values():
            href = service_provider.prepare_href(rendition['href'])
            if rendition['rendition'] == 'baseImage':
                tasks.append(archive_media.s(task_id, guid, href))
            else:
                tasks.append(archive_rendition.s(task_id, guid, rendition['rendition'], href))

        update_status(*finish_subtask_from_progress(task_id))
        if tasks:
            chord((task for task in tasks), update_item.s(crt_task_id == task_id, task_id, guid)).delay()
        elif task_id == crt_task_id:
            update_status(*finish_task_for_progress(task_id))
    except Exception:
        logger.error(traceback.format_exc())


def ingest_set_archived(guid):
    ingest_doc = superdesk.get_resource_service('ingest').find_one(req=None, _id=guid)
    if ingest_doc:
        superdesk.get_resource_service('ingest').patch(ingest_doc.get('_id'), {'archived': utcnow()})


class ArchiveIngestResource(Resource):
    resource_methods = ['POST']
    item_methods = ['GET']
    datasource = {
        'search_backend': 'elastic',
        'facets': facets
    }
    additional_lookup = {
        'url': 'regex("[\w-]+")',
        'field': 'task_id'
    }
    schema = {
        'guid': {
            'type': 'string',
            'required': True,
        },
        'task_id': {
            'type': 'string',
            'required': False,
        }
    }


class ArchiveIngestService(BaseService):

    def create(self, docs, **kwargs):
        for doc in docs:
            ingest_doc = superdesk.get_resource_service('ingest').find_one(req=None, _id=doc.get('guid'))
            if not ingest_doc:
                msg = 'Fail to found ingest item with guid: %s' % doc.get('guid')
                raise superdesk.SuperdeskError(payload=msg)
            ingest_set_archived(doc.get('guid'))

            archived_doc = superdesk.get_resource_service('archive').find_one(req=None, guid=doc.get('guid'))
            if not archived_doc:
                doc.setdefault('_id', doc.get('guid'))
                doc.setdefault('user', str(getattr(flask.g, 'user', {}).get('_id')))
                superdesk.get_resource_service('archive').post([doc])

            task = archive_item.delay(doc.get('guid'), ingest_doc.get('ingest_provider'), doc.get('user'))
            doc['task_id'] = task.id
            if task.state not in ('PROGRESS', states.SUCCESS, states.FAILURE) and not task.result:
                update_status(task.id, 0, 0)

            superdesk.get_resource_service('archive').patch(doc.get('guid'), {"task_id": task.id})

        return [doc.get('guid') for doc in docs]

    def update(self, id, updates):
        pass

    def delete(self, lookup):
        pass

    def find_one(self, req=None, **lookup):
        try:
            task_id = lookup["task_id"]
            task = AsyncResult(task_id)

            if task.state in ('PROGRESS', states.SUCCESS, states.FAILURE) and task.result:
                doc = task.result
            else:
                doc = {}

            doc['state'] = task.state
            doc['task_id'] = task_id
            doc['_id'] = task_id
            doc['_created'] = doc['_updated'] = utcnow()

            return doc
        except Exception:
            msg = 'No progress information is available for task_id: %s' % task_id
            raise superdesk.SuperdeskError(payload=msg)



"""Superdesk"""

import logging
from flask import abort, json, Blueprint, current_app as app  # noqa
from flask.ext.script import Command, Option  # noqa @UnresolvedImport
from eve.methods.common import document_link  # noqa
from .datalayer import SuperdeskDataLayer  # noqa
from werkzeug.exceptions import HTTPException
from eve.utils import config  # noqa
from .eve_backend import EveBackend


API_NAME = 'Superdesk API'
VERSION = (0, 0, 1)
DOMAIN = {}
COMMANDS = {}
BLUEPRINTS = []
app_components = dict()
app_models = dict()
resources = dict()
eve_backend = EveBackend()
available_preferences = dict()


logger = logging.getLogger(__name__)


class SuperdeskError(Exception):
    """Base error class for superdesk."""

    # default error status code
    status_code = 400

    def __init__(self, message=None, status_code=None, payload=None):
        """
        :param message: a human readable error description
        :param status_code: response status code
        :param payload: a dict with request issues
        """
        Exception.__init__(self)
        self.message = message

        if status_code:
            self.status_code = status_code

        if payload:
            self.payload = payload

    def to_dict(self):
        """Create dict for json response."""
        rv = {}
        rv[app.config['STATUS']] = app.config['STATUS_ERR']
        rv['_message'] = self.message or ''
        if hasattr(self, 'payload'):
            rv[app.config['ISSUES']] = self.payload
        return rv


def get_headers(self, environ=None):
    """Fix CORS for abort responses.

    todo(petr): put in in custom flask error handler instead
    """
    return [
        ('Content-Type', 'text/html'),
        ('Access-Control-Allow-Origin', '*'),
        ('Access-Control-Allow-Headers', '*'),
    ]

setattr(HTTPException, 'get_headers', get_headers)


def domain(resource, res_config):
    """Register domain resource"""
    DOMAIN[resource] = res_config


def command(name, command):
    """Register command"""
    COMMANDS[name] = command


def blueprint(blueprint, **kwargs):
    """Register blueprint"""
    blueprint.kwargs = kwargs
    BLUEPRINTS.append(blueprint)


def get_backend():
    """Returns the available backend, this will be changed in a factory if needed."""
    return eve_backend


def get_resource_service(resource_name):
    return resources[resource_name].service


from .app_prepopulate import AppPrepopulateCommand  # noqa


def register_available_preference(preference_name, preference):
    available_preferences[preference_name] = preference



"""Simple NITF parser"""

from datetime import datetime
import xml.etree.ElementTree as etree

ITEM_CLASS = 'text'

subject_fields = ('tobject.subject.type', 'tobject.subject.matter', 'tobject.subject.detail')


def get_norm_datetime(tree):
    return datetime.strptime(tree.attrib['norm'], '%Y%m%dT%H%M%S')


def get_content(tree):
    elements = []
    for elem in tree.find('body/body.content'):
        elements.append(etree.tostring(elem, encoding='UTF-8').decode('utf-8'))
    return ''.join(elements)


def get_keywords(tree):
    keywords = []
    for elem in tree.findall('head/meta'):
        if elem.get('name') == 'anpa-keyword':
            keywords.append(elem.get('content'))
    return keywords


def get_subjects(tree):
    subjects = []
    for elem in tree.findall('head/tobject/tobject.subject'):
        qcode = elem.get('tobject.subject.refnum')
        for field in subject_fields:
            if elem.get(field):
                subjects.append({
                    'name': elem.get(field)
                })

        if len(subjects):
            subjects[-1]['code'] = qcode

    return subjects


def parse(text):
    item = {}
    tree = etree.fromstring(text.encode('utf-8'))
    docdata = tree.find('head/docdata')

    item['type'] = ITEM_CLASS
    item['guid'] = item['uri'] = docdata.find('doc-id').get('id-string')
    item['urgency'] = docdata.find('urgency').get('ed-urg', '5')
    item['firstcreated'] = get_norm_datetime(docdata.find('date.issue'))
    item['versioncreated'] = get_norm_datetime(docdata.find('date.issue'))
    item['keywords'] = get_keywords(tree)
    item['subject'] = get_subjects(tree)
    item['body_html'] = get_content(tree)
    item['pubstatus'] = docdata.attrib.get('management-status', 'usable')

    item['headline'] = tree.find('body/body.head/hedline/hl1').text

    try:
        item['copyrightholder'] = docdata.find('doc.copyright').get('holder')
    except AttributeError:
        pass

    return item



"""Superdesk push notifications"""

import logging
import asyncio

from flask import current_app as app
from flask import json
from datetime import datetime


logger = logging.getLogger(__name__)


def init_app(app):
    from autobahn.asyncio.websocket import WebSocketClientProtocol
    from autobahn.asyncio.websocket import WebSocketClientFactory
    from ws import host, port

    class Client(WebSocketClientProtocol):

        def __init__(self, *args):
            WebSocketClientProtocol.__init__(self, *args)
            self.opened = asyncio.Future()

        def notify(self, **kwargs):
            kwargs.setdefault('_created', datetime.utcnow().isoformat())
            self.sendMessage(json.dumps(kwargs).encode('utf8'))

        def onConnect(self, response):
            self.opened.set_result(None)

    factory = WebSocketClientFactory()
    factory.protocol = Client
    try:
        loop = asyncio.get_event_loop()
        coro = loop.create_connection(factory, host, port)
        _transport, client = loop.run_until_complete(coro)
        loop.run_until_complete(client.opened)
        app.notification_client = client
    except OSError:  # no ws server running
        pass


def push_notification(name, **kwargs):
    logger.info('pushing event {0} ({1})'.format(name, json.dumps(kwargs)))
    if app.notification_client:
        app.notification_client.notify(event=name, extra=kwargs)



from superdesk.activity import add_activity
from eve.utils import ParsedRequest
from flask import g, render_template
from settings import ADMINS

from superdesk.emails import send_email

import re
import superdesk


def get_users_mentions(text):
    pattern = re.compile("(^|\s)\@([a-zA-Z0-9-_.]\w+)")
    usernames = set(username for match in re.finditer(pattern, text) for username in match.groups())
    return list(usernames)


def send_user_mentioned_email(recipients, user_name, doc, url):
    print('sending mention email to:', recipients)
    send_email.delay(subject='You were mentioned in a comment by %s' % user_name,
                     sender=ADMINS[0],
                     recipients=recipients,
                     text_body=render_template("user_mention.txt", text=doc['text'], username=user_name, link=url),
                     html_body=render_template("user_mention.html", text=doc['text'], username=user_name, link=url))


def send_email_to_mentioned_users(doc, mentioned_users, origin):
    prefs_service = superdesk.get_resource_service('preferences')
    recipients = []
    for user in mentioned_users:
        send_email = prefs_service.email_notification_is_enabled(user)
        if send_email:
            user_doc = superdesk.get_resource_service('users').find_one(req=None, _id=user)
            recipients.append(user_doc['email'])
    if recipients:
        username = g.user.get('display_name') or g.user.get('username')
        url = '{}/#/authoring/{}?comments={}'.format(origin, doc['item'], doc['_id'])
        send_user_mentioned_email(recipients, username, doc, url)


def get_users(usernames):
    req = ParsedRequest()
    users = superdesk.get_resource_service('users').get(req=req, lookup={'username': {'$in': usernames}})
    users = {user.get('username'): user.get('_id') for user in users}
    return users


def notify_mentioned_users(docs, origin):
    for doc in docs:
        mentioned_users = doc.get('mentioned_users', {}).values()
        add_activity('', type='comment', item=str(doc.get('item')),
                     comment=doc.get('text'), comment_id=str(doc.get('_id')),
                     notify=mentioned_users)
        send_email_to_mentioned_users(doc, mentioned_users, origin)



import superdesk
from eve.io.base import DataLayer
from eve.io.mongo import Mongo
from eve.utils import config, ParsedRequest
from eve_elastic import Elastic
from .utils import import_by_path
from pyelasticsearch.client import JsonEncoder
from bson.objectid import ObjectId
from flask import current_app as app


class SuperdeskJsonEncoder(JsonEncoder):
    '''Customize the JSON encoder used in Elastic'''
    def default(self, value):
        """Convert more Python data types to ES-understandable JSON."""
        if isinstance(value, ObjectId):
            return str(value)
        return super(SuperdeskJsonEncoder, self).default(value)


class SuperdeskDataLayer(DataLayer):
    """Superdesk Data Layer"""

    serializers = Mongo.serializers
    serializers.update(Elastic.serializers)

    def init_app(self, app):
        self.mongo = Mongo(app)
        self.elastic = Elastic(app)
        self.elastic.es.json_encoder = SuperdeskJsonEncoder

        if 'DEFAULT_FILE_STORAGE' in app.config:
            self.storage = import_by_path(app.config['DEFAULT_FILE_STORAGE'])()
            self.storage.init_app(app)
        else:
            self.storage = self.driver

    def find(self, resource, req, lookup):
        return superdesk.get_resource_service(resource).get(req=req, lookup=lookup)

    def find_all(self, resource, max_results=1000):
        req = ParsedRequest()
        req.max_results = max_results
        return self._backend(resource).find(resource, req, None)

    def find_one(self, resource, req, **lookup):
        return superdesk.get_resource_service(resource).find_one(req=req, **lookup)

    def find_one_raw(self, resource, _id):
        return self._backend(resource).find_one_raw(resource, _id)

    def find_list_of_ids(self, resource, ids, client_projection=None):
        return self._backend(resource).find_list_of_ids(resource, ids, client_projection)

    def insert(self, resource, docs, **kwargs):
        return superdesk.get_resource_service(resource).create(docs, **kwargs)

    def update(self, resource, id_, updates):
        return superdesk.get_resource_service(resource).update(id=id_, updates=updates)

    def update_all(self, resource, query, updates):
        datasource = self._datasource(resource)
        driver = self._backend(resource).driver
        collection = driver.db[datasource[0]]
        return collection.update(query, {'$set': updates}, multi=True)

    def replace(self, resource, id_, document):
        return superdesk.get_resource_service(resource).replace(id=id_, document=document)

    def remove(self, resource, lookup=None):
        if lookup is None:
            lookup = {}
        return superdesk.get_resource_service(resource).delete(lookup=lookup)

    def is_empty(self, resource):
        return self._backend(resource).is_empty(resource)

    def _search_backend(self, resource):
        if resource.endswith(app.config['VERSIONS']):
            return
        datasource = self._datasource(resource)
        backend = config.SOURCES[datasource[0]].get('search_backend', None)
        return getattr(self, backend) if backend is not None else None

    def _backend(self, resource):
        datasource = self._datasource(resource)
        backend = config.SOURCES[datasource[0]].get('backend', 'mongo')
        return getattr(self, backend)
